use crate::runtime::{
    Reference,
    acquire, release,
    type_key,
    sequence_length,
    sequence_clear, sequence_reserve,
    sequence_get,
    sequence_insert,
};
use crate::tag;
use super::sequence;

pub struct Sequence {
    managed:bool,
    addr:Reference,
}
impl Sequence {
    pub fn new() -> Self
    {
        Self {
            managed:true,
            addr:unsafe {acquire(sequence())},
        }
    }

    pub fn from(addr:Reference) -> Result<Self,()>
    {
        return if(unsafe {type_key(addr.class)} == tag::SEQUENCE) {
            Ok(Self { managed:false, addr:addr })
        }
        else {
            Err(())
        }
    }

    pub fn with(data:&str) -> Self
    {
        let mut obj = Self::new();
        obj.set(data);
        return obj;
    }

    pub fn set(&mut self, data:&str)
    {
        unsafe { sequence_clear(self.addr); }
        let bytes = Vec::from(data.as_bytes());
        if bytes.len() > 0 {
            unsafe { sequence_reserve(self.addr, bytes.len()); }
            for i in 0..bytes.len() {
                unsafe { sequence_insert(self.addr, i, bytes[i]); }
            }
        }
    }

    pub fn get(&self) -> String
    {
        let length = unsafe {sequence_length(self.addr)};
        if length > 0 {
            let mut bytes = Vec::<u8>::with_capacity(length);
            for i in 0..length {
                bytes.push(unsafe {sequence_get(self.addr, i)});
            }
            return match String::from_utf8(bytes) {
                Ok(data) => data,
                Err(_) => String::new(),
            }
        }
        else {
            return String::new();
        }
    }
}
impl std::ops::Deref for Sequence {
    type Target = Reference;
    fn deref(&self) -> &Self::Target { return &self.addr; }
}
impl Drop for Sequence {
    fn drop(&mut self) { if self.managed { unsafe {release(self.addr)}; } }
}
